@@ -3,7 +3,7 @@ package Net::AS2;
 use strict;
 use warnings;
 use autodie qw(:file :filesys);
-our $VERSION = '1.001'; # TRIAL VERSION
+our $VERSION = '1.0111'; # TRIAL VERSION
 
 =head1 NAME
 
@@ -480,11 +480,19 @@ sub decode_message
     croak 'content is undefined'
       unless defined $content;
 
-    my $message_id = $headers->header('Message-Id');
-    my $async_url  = $headers->header('Receipt-Delivery-Option');
-    my @new_prefix = ($message_id, $async_url, 0);
+    my @new_prefix = (
+        scalar($headers->header('Message-Id')),
+        scalar($headers->header('Receipt-Delivery-Option')),
+        0
+    );
 
-    if (defined $async_url && $async_url !~ m{^https?://}) {
+    # Validate Message-Id format
+    eval {
+        $new_prefix[0] = $self->get_message_id($new_prefix[0]);
+    } or do {
+        return Net::AS2::Message->create_error_message(@new_prefix, 'unexpected-processing-error', "Malformed AS2 Message, $@.");
+    };
+    if (defined($new_prefix[1]) && $new_prefix[1] !~ m{^https?://}) {
         $new_prefix[1] = undef;
         return Net::AS2::Message->create_failure_message(@new_prefix, 'Async transport other than http/https is not supported');
     }
@@ -503,7 +511,7 @@ sub decode_message
 
     unless (
         defined $content_type &&
-        defined $message_id   &&
+        defined $new_prefix[0] &&
         defined $version &&
         defined $from &&
         defined $to)
@@ -579,7 +587,8 @@ sub decode_message
         unless defined $bh;
 
     $content = $bh->as_string;
-    return Net::AS2::Message->new(@new_prefix, $mic, $content, $self->{Signature});
+    my $filename = $entity->head->mime_attr('content-disposition.filename');
+    return Net::AS2::Message->new(@new_prefix, $mic, $content, $self->{Signature}, $filename);
 }
 
 =item $mdn = $as2->decode_mdn($headers, $content)
@@ -745,6 +754,11 @@ Message id of this request should be supplied, or a random one would be generate
 
 Content type of the message should be supplied.
 
+=item Filename
+
+I<Optional.>
+Sets the Content-Disposition filename. Default is "payload".
+
 =back
 
 In case of HTTP failure, the MDN object will be marked with C<$mdn-E<gt>is_error>.
@@ -770,8 +784,9 @@ sub send ## no critic (ProhibitBuiltinHomonyms)
 
     my $message_id = $self->get_message_id($opts{MessageId}, generate => 1);
 
+    my $filename = delete $opts{Filename} // 'payload';
     $opts{Encoding} = 'base64';
-    $opts{Disposition} //= 'attachment';
+    $opts{Disposition} //= qq{attachment; filename="$filename"};
     $opts{Subject} //= 'AS2 Message';
     $opts{'X-Mailer'} = undef;
 
@@ -822,7 +837,7 @@ sub _send_preprocess
 
     push @header, (
         defined $target_url ? ('Recipient-Address' => $target_url) : (),
-        'Message-Id'  => $message_id,
+        'Message-Id'  => "<$message_id>",
         'AS2-Version' => '1.0',
         'AS2-From'    => $self->encode_as2_id($self->{MyId}),
         'AS2-To'      => $self->encode_as2_id($self->{PartnerId}),
@@ -1020,7 +1035,7 @@ sub _base64_digest {
     while (length($digest) % 4) {
         $digest .= '=';
     }
-			
+
     return $digest;
 }
 
